use crate::{amount::AmountCents, asset::AssetId, Amount, Asset, Error};
use std::{collections::HashMap, sync::Arc};

#[derive(Clone, Debug, Eq, PartialEq)]
pub struct AssetManager {
    assets: HashMap<AssetId, AssetDefinition>,
}

#[derive(Clone, Debug, Eq, PartialEq, serde::Deserialize)]
pub struct AssetDefinition {
    #[serde(flatten)]
    asset: Asset,
    name: Arc<str>,
}

impl AssetManager {
    pub fn new(assets: Vec<AssetDefinition>) -> Self {
        Self {
            assets: assets
                .into_iter()
                .map(|asset| (asset.asset.id, asset))
                .collect(),
        }
    }

    pub fn asset(&self, id: AssetId) -> Result<Asset, Error> {
        self.assets
            .get(&id)
            .map(|asset| asset.asset)
            .ok_or(Error::AssetNotFound(id))
    }

    pub fn asset_name(&self, asset: &Asset) -> Result<Arc<str>, Error> {
        self.assets
            .get(&asset.id)
            .map(|asset| asset.name.clone())
            .ok_or(Error::AssetNotFound(asset.id))
    }

    pub fn amount(&self, id: AssetId, cents: AmountCents) -> Result<Amount, Error> {
        self.asset(id).map(|asset| Amount::new(asset, cents))
    }
}
