use crate::{amount::AmountCents, storage, Asset, Status};

#[derive(thiserror::Error, Debug)]
pub enum Error {
    #[error("Payment {0} is already spent")]
    SpentPayment(usize),

    #[error(
        "Payment {0} is not a valid amount. Spending {1} and issuing {2}. Both amounts must match"
    )]
    InvalidAmount(Asset, AmountCents, AmountCents),

    #[error("Missing input payment for asset {0}")]
    MissingSpendingAsset(Asset),

    #[error("Missing output payment for asset {0}")]
    MissingPaymentAsset(Asset),

    #[error("Transaction updates are not allowed")]
    TransactionUpdatesNotAllowed,

    #[error("Mismatch between spend and create payment assets")]
    MismatchSpendingPayingAssets,

    #[error("Status transition from {0} to {1} is not allowed")]
    StatusTransitionNotAllowed(Status, Status),

    #[error("Storage: {0}")]
    Storage(#[from] storage::Error),

    #[error("Overflow")]
    Overflow,
}
